@@ -1,25 +1,25 @@
 /*----------------------------------------------------------------------------
- ChucK Concurrent, On-the-fly Audio Programming Language
-   Compiler and Virtual Machine
+  ChucK Strongly-timed Audio Programming Language
+    Compiler and Virtual Machine
 
- Copyright (c) 2003 Ge Wang and Perry R. Cook.  All rights reserved.
-   http://chuck.stanford.edu/
-   http://chuck.cs.princeton.edu/
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
+    http://chuck.stanford.edu/
+    http://chuck.cs.princeton.edu/
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- U.S.A.
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  U.S.A.
  -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
@@ -39,6 +39,7 @@
 #ifndef __DISABLE_OTF_SERVER__
 #include "util_thread.h"
 #endif
+
 #include <map>
 
 
@@ -95,17 +96,16 @@ struct Chuck_Carrier
         compiler( NULL ),
         env( NULL ),
         vm( NULL ),
+        chout( NULL ),
+        cherr( NULL ),
         #ifndef __DISABLE_OTF_SERVER__
         otf_socket( NULL ),
         otf_port( 0 ),
         otf_thread( 0 ),
         #endif
         #ifndef __DISABLE_WVOUT__
-        stk_writeThread( NULL ),
+        stk_writeThread( NULL )
         #endif
-        chout( NULL ),
-        cherr( NULL )
-
     { }
 
     // get hint: is realtime audio?
