@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
+  ChucK Strongly-timed Audio Programming Language
     Compiler and Virtual Machine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
@@ -43,6 +43,8 @@
 #include "chuck_type.h"
 #include "chuck_ugen.h"
 #include <string>
+#include <map>
+#include <vector>
 
 // forward reference for ChucK VM
 struct Chuck_VM;
@@ -73,6 +75,7 @@ struct Chuck_Set_Global_Float_Array_Value_Request;
 struct Chuck_Get_Global_Float_Array_Value_Request;
 struct Chuck_Set_Global_Associative_Float_Array_Value_Request;
 struct Chuck_Get_Global_Associative_Float_Array_Value_Request;
+struct Chuck_Get_Global_All_Request; // 1.5.1.0
 struct Chuck_Execute_Chuck_Msg_Request;
 
 // forward references for global storage
@@ -86,7 +89,6 @@ struct Chuck_Global_Object_Container;
 
 
 
-
 //-----------------------------------------------------------------------------
 // name: enum Global_Request_Type
 // desc: what kind of global message request is this? (REFACTOR-2017)
@@ -119,6 +121,8 @@ enum Chuck_Global_Request_Type
     get_global_float_array_value_request,
     set_global_associative_float_array_value_request,
     get_global_associative_float_array_value_request,
+    // 1.5.1.0 (ge) get all global variables
+    get_global_all_request,
     // shreds
     spork_shred_request,
     // chuck_msg
@@ -161,6 +165,8 @@ struct Chuck_Global_Request
         Chuck_Get_Global_Float_Array_Value_Request* getFloatArrayValueRequest;
         Chuck_Set_Global_Associative_Float_Array_Value_Request* setAssociativeFloatArrayValueRequest;
         Chuck_Get_Global_Associative_Float_Array_Value_Request* getAssociativeFloatArrayValueRequest;
+        // get all global variables | 1.5.1.0
+        Chuck_Get_Global_All_Request* getAllRequest;
         // shreds
         Chuck_VM_Shred* shred;
         // chuck_msg
@@ -178,6 +184,28 @@ struct Chuck_Global_Request
 
 
 
+//-----------------------------------------------------------------------------
+// name: struct Chuck_Globals_TypeValue | 1.5.1.0 (ge) added
+// desc: a type-value pair representing a global value in the system
+//       used to retrive a list of all current global variables
+//-----------------------------------------------------------------------------
+struct Chuck_Globals_TypeValue
+{
+    // type string of global variable
+    std::string type;
+    // global variable name
+    std::string name;
+
+    // constructor
+    Chuck_Globals_TypeValue()
+    { }
+    Chuck_Globals_TypeValue( const std::string & t, const std::string & n )
+    : type(t), name(n) { }
+};
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: struct Chuck_Globals_Manager
 // desc: manager for globals storage | added 1.4.1.0 (jack)
@@ -243,6 +271,10 @@ public:
     t_CKBOOL getGlobalAssociativeFloatArrayValue( const char * name, const char * key, void (*callback)(const char*, t_CKFLOAT) );
     t_CKBOOL getGlobalAssociativeFloatArrayValue( const char * name, t_CKINT callbackID, const char * key, void (*callback)(t_CKINT, t_CKFLOAT) );
 
+    // 1.5.1.0 (ge) get all global variables
+    t_CKBOOL getAllGlobalVariables( void (*callback)( const std::vector<Chuck_Globals_TypeValue> & list, void * data ),
+                                    void * data = NULL );
+
 public:
     // run Chuck_Msg in the globals order
     t_CKBOOL execute_chuck_msg_with_globals( Chuck_Msg* msg );
@@ -250,7 +282,7 @@ public:
 public:
     // REFACTOR-2017: externally accessible + global variables.
     // WARNING: these internal functions are to be used only by other
-    // chuck code in the audio thread.
+    // chuck code in the audio thread...
     t_CKBOOL init_global_int( const std::string & name );
     t_CKINT get_global_int_value( const std::string & name );
     t_CKINT* get_ptr_to_global_int( const std::string & name );
@@ -287,6 +319,9 @@ public:
     Chuck_Object* get_global_object( const std::string & name );
     Chuck_Object** get_ptr_to_global_object( const std::string & name );
 
+    // 1.5.1.0 (ge) get all global variables
+    void get_all_global_variables( std::vector<Chuck_Globals_TypeValue> & list );
+
     t_CKBOOL should_call_global_ctor( const std::string & name, te_GlobalType type );
     void global_ctor_was_called( const std::string & name, te_GlobalType type );
 
