@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
+  ChucK Strongly-timed Audio Programming Language
     Compiler and Virtual Machine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
@@ -37,10 +37,10 @@
 // modified: Ge Wang (gewang@cs.princeton.edu)
 // date: Autumn 2002
 //-----------------------------------------------------------------------------
+#include "chuck_table.h"
+#include "chuck_symbol.h"
 #include <stdio.h>
 #include <string.h>
-#include "chuck_table.h"
-#include "chuck_utils.h"
 
 #define TABSIZE 65437
 
