@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
+  ChucK Strongly-timed Audio Programming Language
     Compiler and Virtual Machine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
@@ -38,9 +38,8 @@
 #include <float.h>
 #include <stdlib.h>
 #include <time.h>
-
 #include <limits.h>
-#include <limits>
+
 #include <vector> // 1.5.0.0 (ge) | added
 
 
@@ -726,26 +725,13 @@ CK_DLL_SFUN( isnan_impl )
 
 // equal( x, y ) -- 1.4.1.1 (added ge)
 // returns whether x and y (floats) are considered equal
-//
-// Knuth, Donald E., /The Art of Computer Programming
-// Volume II: Seminumerical Algorithms/, Addison-Wesley, 1969.
-// based on Knuth section 4.2.2 pages 217-218
-// https://www.cs.technion.ac.il/users/yechiel/c++-faq/floating-point-arith.html
 CK_DLL_SFUN( equal_impl )
 {
-    const t_CKFLOAT epsilon = .00000001; // a small number 1e-8
     // get arguments
     t_CKFLOAT x = GET_CK_FLOAT(ARGS);
     t_CKFLOAT y = *((t_CKFLOAT *)ARGS + 1);
-    // absolute values
-    t_CKFLOAT abs_x = (x >= 0.0 ? x : -x);
-    t_CKFLOAT abs_y = (y >= 0.0 ? y : -y);
-    // smaller of the two absolute values (this step added by ge; ensures symmetry)
-    t_CKFLOAT min = abs_x < abs_y ? abs_x : abs_y;
-    // absolute value of the difference
-    t_CKFLOAT v = x-y; t_CKFLOAT abs_v = (v >= 0.0 ? v : -v);
-    // test whether difference is less/equal to episilon * smaller of two abs values
-    RETURN->v_int = (abs_v <= (epsilon * min));
+    // equal
+    RETURN->v_int = ck_equals(x,y);
 }
 
 // floatMax
@@ -829,8 +815,8 @@ CK_DLL_SFUN( phase_impl )
 CK_DLL_SFUN( rtop_impl )
 {
     // get array
-    Chuck_Array16 * from = (Chuck_Array16 *)GET_NEXT_OBJECT(ARGS);
-    Chuck_Array16 * to = (Chuck_Array16 *)GET_NEXT_OBJECT(ARGS);
+    Chuck_ArrayVec2 * from = (Chuck_ArrayVec2 *)GET_NEXT_OBJECT(ARGS);
+    Chuck_ArrayVec2 * to = (Chuck_ArrayVec2 *)GET_NEXT_OBJECT(ARGS);
 
     // make sure not null
     if( !from || !to )
@@ -868,8 +854,8 @@ CK_DLL_SFUN( rtop_impl )
 CK_DLL_SFUN( ptor_impl )
 {
     // get array
-    Chuck_Array16 * from = (Chuck_Array16 *)GET_NEXT_OBJECT(ARGS);
-    Chuck_Array16 * to = (Chuck_Array16 *)GET_NEXT_OBJECT(ARGS);
+    Chuck_ArrayVec2 * from = (Chuck_ArrayVec2 *)GET_NEXT_OBJECT(ARGS);
+    Chuck_ArrayVec2 * to = (Chuck_ArrayVec2 *)GET_NEXT_OBJECT(ARGS);
 
     // make sure not null
     if( !from || !to )
