@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------
-  ChucK Concurrent, On-the-fly Audio Programming Language
+  ChucK Strongly-timed Audio Programming Language
     Compiler and Virtual Machine
 
-  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+  Copyright (c) 2003 Ge Wang and Perry R. Cook. All rights reserved.
     http://chuck.stanford.edu/
     http://chuck.cs.princeton.edu/
 
@@ -170,10 +170,6 @@ DLL_QUERY libstd_query( Chuck_DL_Query * QUERY )
     // set name
     QUERY->setname( QUERY, "Std" );
 
-    /*! \example
-    std.rand2f( 100.0, 1000.0 ) => stdout;
-    */
-
     // register deprecate
     type_engine_register_deprecate( env, "std", "Std" );
 
@@ -745,9 +741,9 @@ CK_DLL_SFUN( system_impl )
     else
     {
         // log
-        EM_log( CK_LOG_SEVERE, "invoking system( CMD )..." );
+        EM_log( CK_LOG_HERALD, "invoking system( CMD )..." );
         EM_pushlog();
-        EM_log( CK_LOG_SEVERE, "CMD: \"%s\"", cmd );
+        EM_log( CK_LOG_HERALD, "CMD: \"%s\"", cmd );
         EM_poplog();
         RETURN->v_int = system( cmd );
     }
@@ -1076,7 +1072,7 @@ t_CKBOOL KBHitManager::init()
     the_buf = new CBufferAdvance;
     if( !the_buf->initialize( BUFFER_SIZE, sizeof(t_CKINT) ) )
     {
-        EM_log( CK_LOG_SEVERE, "KBHitManager: couldn't allocate central KB buffer..." );
+        EM_log( CK_LOG_HERALD, "KBHitManager: couldn't allocate central KB buffer..." );
         CK_SAFE_DELETE( the_buf );
         return FALSE;
     }
